/*
-----------------------------------------------------------------------------
This source file is part of OGRE
(Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org

Copyright (c) 2000-2014 Torus Knot Software Ltd
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
-----------------------------------------------------------------------------
*/

//-----------------------------------------------------------------------------
// Transform the output position to the current "monitor"
//-----------------------------------------------------------------------------

void SGX_InstancedViewportsTransform(
                in float4 i_position,
                in float4x4	i_worldViewMatrix,
                in float4x4	i_projectionMatrix,
						    in  float4 i_viewportOffsetMatrixR0,   
						    in  float4 i_viewportOffsetMatrixR1,   
						    in  float4 i_viewportOffsetMatrixR2,   
						    in  float4 i_viewportOffsetMatrixR3,   
						    in  float2 i_monitorsCount,   
						    in float4 i_monitorIndex, 
						    out float4 o_position)
 {
   o_position = mul(i_worldViewMatrix, i_position);
   float4x4 viewportOffset = float4x4(i_viewportOffsetMatrixR0, 
                                      i_viewportOffsetMatrixR1, 
                                      i_viewportOffsetMatrixR2, 
                                      i_viewportOffsetMatrixR3
                                     );
   o_position = mul(viewportOffset, o_position);
   o_position = mul(i_projectionMatrix, o_position);
   float2 monitorIndexNorm =i_monitorIndex.xy  - ((i_monitorsCount  - 1.0)/ 2.0) ;
   o_position.xy = 
   (o_position.xy + (o_position.w * monitorIndexNorm)*2.0)  / i_monitorsCount ;
 };

//-----------------------------------------------------------------------------
// Discard any pixel that is outside the bounds of the current "monitor"
//-----------------------------------------------------------------------------

void SGX_InstancedViewportsDiscardOutOfBounds(
						    in  float2 i_monitorsCount,   
						    in float4 i_monitorIndex, 
						    in float4 i_positionProjectiveSpace)
{
   float2 boxedXY = i_positionProjectiveSpace.xy / (i_positionProjectiveSpace.w * 2);
   boxedXY = (boxedXY + 0.5) * i_monitorsCount;
    float2 middleMonitor = ((i_monitorIndex.xy + 0.5));
    
    boxedXY = abs(boxedXY - middleMonitor);
   float maxM = max(boxedXY.x,boxedXY.y);
   if (maxM >= 0.5)
   {
      discard;
   }
}